@@ -19,17 +19,25 @@
 #include <config.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
 #include <string.h>
 #include <thunar-vfs/thunar-vfs.h>
 #include <libxfcegui4/libxfcegui4.h>
 #include <libexif/exif-data.h>
 #include <dbus/dbus-glib.h>
 
+#ifdef HAVE_XFCONF
+#include <xfconf/xfconf.h>
+#endif
+
 #include "navigator.h"
 #include "thumbnail_bar.h"
 #include "picture_viewer.h"
 #include "main_window.h"
 
+#define XFDESKTOP_SELECTION_FMT "XFDESKTOP_SELECTION_%d"
+
 
 struct _RsttoMainWindowPriv
 {
@@ -60,6 +68,7 @@ struct _RsttoMainWindowPriv
         gdouble         slideshow_timeout;
         const GdkColor *bg_color;
         gboolean        scale_to_100;
+        RsttoDesktop    desktop;
     } settings;
 
     struct {
@@ -125,6 +134,7 @@ struct _RsttoMainWindowPriv
 
             GtkWidget *menu_item_separator_2;
             GtkWidget *menu_item_fullscreen;
+            GtkWidget *menu_item_set_wallpaper;
         } view;
 
         GtkWidget *menu_item_go;
@@ -189,6 +199,10 @@ static void
 cb_rstto_main_window_toggle_toolbar(GtkWidget *widget, RsttoMainWindow *window);
 static void
 cb_rstto_main_window_toggle_fullscreen(GtkWidget *widget, RsttoMainWindow *window);
+#ifdef WITH_DESKTOP_WALLPAPER
+static void
+cb_rstto_main_window_set_wallpaper(GtkWidget *widget, RsttoMainWindow *window);
+#endif
 static gboolean
 cb_rstto_main_window_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
 static void
@@ -397,6 +411,64 @@ rstto_main_window_init(RsttoMainWindow *window)
 
     window->priv->menus.view.menu_item_fullscreen = gtk_image_menu_item_new_from_stock(GTK_STOCK_FULLSCREEN, NULL);
 
+    GtkWidget *wallpaper_image = gtk_image_new_from_icon_name("preferences-desktop-wallpaper", GTK_ICON_SIZE_MENU);
+    window->priv->menus.view.menu_item_set_wallpaper = gtk_image_menu_item_new_with_mnemonic(_("_Set as wallpaper"));
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(window->priv->menus.view.menu_item_set_wallpaper), wallpaper_image);
+
+    /** FIXME: HACK */
+#ifdef HAVE_XFCONF 
+#ifdef WITH_DESKTOP_WALLPAPER
+    /** Set xfce-desktop as default when support has been compiled in */
+    /* Check if xfdesktop is running */
+    {
+        Atom actual_type;
+        int actual_format;
+        unsigned long n_items;
+        unsigned long bytes_after;
+        unsigned char *prop;
+        gchar selection_name[100];
+
+        GdkScreen *gdk_screen = gdk_screen_get_default();
+        gint xscreen = gdk_screen_get_number(gdk_screen);
+
+
+        g_snprintf(selection_name, 100, XFDESKTOP_SELECTION_FMT, xscreen);
+
+        Window root_window = GDK_ROOT_WINDOW();
+        Atom xfce_desktop_atom = XInternAtom (gdk_display, "XFCE_DESKTOP_WINDOW", False);
+        Atom xfce_selection_atom = XInternAtom (gdk_display, selection_name, False);
+        XGetWindowProperty (gdk_display, 
+                            root_window,
+                            xfce_desktop_atom,
+                            0,
+                            1,
+                            False,
+                            AnyPropertyType,
+                            &actual_type,
+                            &actual_format,
+                            &n_items,
+                            &bytes_after,
+                            &prop);
+        if (n_items > 0)
+        {
+            /* TODO: check XID */
+            /* Window xid = (Window) prop[1]; */
+            if(XGetSelectionOwner(gdk_display, xfce_selection_atom) == prop[1] != 0)
+                window->priv->settings.desktop = RSTTO_DESKTOP_XFCE;
+            else
+            {
+                g_debug("xfdesktop is not running");
+            }
+        }
+        else
+        {
+            g_debug("xfdesktop is not running");
+        }
+    }
+#endif
+#endif
+
+
     gtk_widget_add_accelerator(window->priv->menus.view.menu_item_fullscreen, "activate", accel_group, GDK_F11, 0,GTK_ACCEL_VISIBLE);
     gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(window->priv->menus.view.menu_item_show_toolbar), TRUE);
 
@@ -407,6 +479,7 @@ rstto_main_window_init(RsttoMainWindow *window)
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_rotate);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_separator_2);
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_fullscreen);
+    gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.view.menu), window->priv->menus.view.menu_item_set_wallpaper);
 
 /* Create 'View/Show thumbnail-bar' menu */
     window->priv->menus.view.show_thumbnail_viewer.menu = gtk_menu_new();
@@ -512,6 +585,8 @@ rstto_main_window_init(RsttoMainWindow *window)
     gtk_widget_set_sensitive(window->priv->menus.go.menu_item_previous, FALSE);
     gtk_widget_set_sensitive(window->priv->menus.go.menu_item_play, FALSE);
 
+    gtk_widget_set_sensitive(window->priv->menus.view.menu_item_set_wallpaper, FALSE);
+
 /* Create 'Help' menu */
     window->priv->menus.menu_item_help = gtk_menu_item_new_with_mnemonic(_("_Help"));
     gtk_menu_shell_append(GTK_MENU_SHELL(window->priv->menus.menu), window->priv->menus.menu_item_help);
@@ -674,6 +749,13 @@ rstto_main_window_init(RsttoMainWindow *window)
             "activate",
             G_CALLBACK(cb_rstto_main_window_toggle_fullscreen), window);
 
+    /* Set Wallpaper */
+#ifdef WITH_DESKTOP_WALLPAPER
+    g_signal_connect(window->priv->menus.view.menu_item_set_wallpaper, 
+            "activate",
+            G_CALLBACK(cb_rstto_main_window_set_wallpaper), window);
+#endif
+
     /* Play / Pause */
     g_signal_connect(window->priv->menus.go.menu_item_play, 
             "activate",
@@ -979,7 +1061,7 @@ void
 rstto_main_window_set_max_cache_size (RsttoMainWindow *window, gint max_cache_size)
 {
     window->priv->settings.max_cache_size = max_cache_size;
-    rstto_navigator_set_max_history_size(window->priv->navigator, max_cache_size * 1000000);
+    rstto_navigator_set_max_history_size(window->priv->navigator, (guint64)max_cache_size * 1000000);
 }
 
 void
@@ -1049,6 +1131,57 @@ cb_rstto_main_window_toggle_toolbar(GtkWidget *widget, RsttoMainWindow *window)
     }
 }
 
+#ifdef WITH_DESKTOP_WALLPAPER
+static void
+cb_rstto_main_window_set_wallpaper(GtkWidget *widget, RsttoMainWindow *window)
+{
+    RsttoNavigatorEntry *entry = rstto_navigator_get_file(window->priv->navigator);
+    ThunarVfsInfo *info = rstto_navigator_entry_get_info(entry);
+    gchar *path = thunar_vfs_path_dup_string(info->path);
+
+
+    switch (window->priv->settings.desktop)
+    {
+#ifdef HAVE_XFCONF
+        case RSTTO_DESKTOP_XFCE:
+            {
+
+                XfconfChannel *xfdesktop_channel = xfconf_channel_new("xfce4-desktop");
+
+                /*
+                 * Retrieve the screen and monitor number where the main ristretto window is running,
+                 * set the wallpaper there.
+                 */
+                GdkScreen *gdk_screen = gdk_screen_get_default();
+                gint screen = gdk_screen_get_number(gdk_screen);
+                gint monitor = gdk_screen_get_monitor_at_window(gdk_screen, GTK_WIDGET(window)->window);
+
+                gchar *image_path_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-path", screen, monitor);
+                gchar *image_show_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-show", screen, monitor);
+                gchar *image_style_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-style", screen, monitor);
+                if(xfconf_channel_set_string(xfdesktop_channel, image_path_prop, path) == TRUE)
+                {
+                    xfconf_channel_set_bool(xfdesktop_channel, image_show_prop, TRUE);
+                    xfconf_channel_set_int(xfdesktop_channel, image_style_prop, 4);
+                }
+                else
+                {
+                    /** FAILED */
+                }
+                g_free(image_path_prop);
+                g_free(image_show_prop);
+                g_object_unref(xfdesktop_channel);
+            }
+            break;
+#endif
+        default:
+            g_debug("not supported");
+            break;
+    }
+    g_free(path);
+}
+#endif
+
 static gboolean
 cb_rstto_main_window_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 {
@@ -1351,9 +1484,10 @@ cb_rstto_main_window_preferences(GtkWidget *widget, RsttoMainWindow *window)
                 rstto_main_window_set_pv_bg_color(window, NULL);
             }
             rstto_picture_viewer_redraw(RSTTO_PICTURE_VIEWER(window->priv->picture_viewer));
-            rstto_main_window_set_max_cache_size(window, GTK_ADJUSTMENT(cache_adjustment)->value);
+            rstto_main_window_set_max_cache_size(window, (guint)GTK_ADJUSTMENT(cache_adjustment)->value);
 
             window->priv->settings.scale_to_100 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(resize_on_maximize_check));
+
         default:
             break;
     }
@@ -1404,7 +1538,7 @@ static void
 cb_rstto_main_window_open_file(GtkWidget *widget, RsttoMainWindow *window)
 {
     GtkStatusbar *statusbar = GTK_STATUSBAR(window->priv->statusbar);
-    g_object_add_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+    g_object_add_weak_pointer(G_OBJECT(window), (gpointer)statusbar);
 
     gint context_id = gtk_statusbar_get_context_id(statusbar, "StatusMessages");
     gint message_id = gtk_statusbar_push(statusbar, context_id, N_("Opening file(s)..."));
@@ -1436,7 +1570,7 @@ cb_rstto_main_window_open_file(GtkWidget *widget, RsttoMainWindow *window)
     if (statusbar)
     {
         gtk_statusbar_remove(statusbar, context_id, message_id);
-        g_object_remove_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+        g_object_remove_weak_pointer(G_OBJECT(window), (gpointer)statusbar);
     }
 }
 
@@ -1444,7 +1578,7 @@ static void
 cb_rstto_main_window_open_folder(GtkWidget *widget, RsttoMainWindow *window)
 {
     GtkStatusbar *statusbar = GTK_STATUSBAR(window->priv->statusbar);
-    g_object_add_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+    g_object_add_weak_pointer(G_OBJECT(window), (gpointer)statusbar);
 
     gint context_id = gtk_statusbar_get_context_id(statusbar, "StatusMessages");
     gint message_id = gtk_statusbar_push(statusbar, context_id, N_("Opening file(s)..."));
@@ -1473,7 +1607,7 @@ cb_rstto_main_window_open_folder(GtkWidget *widget, RsttoMainWindow *window)
     if (statusbar)
     {
         gtk_statusbar_remove(statusbar, context_id, message_id);
-        g_object_remove_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+        g_object_remove_weak_pointer(G_OBJECT(window), (gpointer)statusbar);
     }
 }
 
@@ -1481,7 +1615,7 @@ static void
 cb_rstto_main_window_open_recent(GtkRecentChooser *chooser, RsttoMainWindow *window)
 {
     GtkStatusbar *statusbar = GTK_STATUSBAR(window->priv->statusbar);
-    g_object_add_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+    g_object_add_weak_pointer(G_OBJECT(window), (gpointer)statusbar);
 
     gint context_id = gtk_statusbar_get_context_id(statusbar, "StatusMessages");
     gint message_id = gtk_statusbar_push(statusbar, context_id, N_("Opening file(s)..."));
@@ -1507,12 +1641,13 @@ cb_rstto_main_window_open_recent(GtkRecentChooser *chooser, RsttoMainWindow *win
             }
         }
         thunar_vfs_path_unref(vfs_path);
+        g_free(path);
     }
 
     if (statusbar)
     {
         gtk_statusbar_remove(statusbar, context_id, message_id);
-        g_object_remove_weak_pointer(G_OBJECT(window), (gpointer *)&statusbar);
+        g_object_remove_weak_pointer(G_OBJECT(window), (gpointer)statusbar);
     }
 }
 
@@ -1598,6 +1733,17 @@ cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoN
         gtk_widget_set_sensitive(window->priv->menus.go.menu_item_play, TRUE);
         gtk_widget_set_sensitive(window->priv->menus.go.menu_item_pause, TRUE);
 
+#ifdef WITH_DESKTOP_WALLPAPER 
+        if (window->priv->settings.desktop > 0)
+        {
+            gtk_widget_set_sensitive(window->priv->menus.view.menu_item_set_wallpaper, TRUE);
+        }
+        else
+        {
+            gtk_widget_set_sensitive(window->priv->menus.view.menu_item_set_wallpaper, FALSE);
+        }
+#endif
+
         gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_next), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(window->priv->toolbar.tool_item_previous), TRUE);
 
@@ -1696,6 +1842,7 @@ cb_rstto_main_window_nav_iter_changed(RsttoNavigator *navigator, gint nr, RsttoN
     else
     {
         gtk_widget_set_sensitive(window->priv->menus.file.menu_item_close, FALSE);
+        gtk_widget_set_sensitive(window->priv->menus.view.menu_item_set_wallpaper, FALSE);
         gtk_window_set_title(GTK_WINDOW(window), PACKAGE_STRING);
         if (rstto_navigator_get_n_files(window->priv->navigator) == 0)
         {
@@ -1933,3 +2080,28 @@ rstto_main_window_get_statusbar(RsttoMainWindow *window)
 {
     return GTK_STATUSBAR(window->priv->statusbar);
 }
+
+gint
+rstto_main_window_get_desktop(RsttoMainWindow *window)
+{
+    return window->priv->settings.desktop;
+}
+
+gint
+rstto_main_window_set_desktop(RsttoMainWindow *window, RsttoDesktop desktop)
+{
+    switch(desktop)
+    {
+#ifdef WITH_DESKTOP_WALLPAPER
+#ifdef HAVE_XFCONF
+        case RSTTO_DESKTOP_XFCE:
+            window->priv->settings.desktop = desktop;
+            break;
+#endif
+#endif
+        default:
+            window->priv->settings.desktop = RSTTO_DESKTOP_NONE;
+            break;
+    }
+    return 0;
+}
