@@ -459,10 +459,8 @@ rstto_thumbnail_bar_realize(GtkWidget *widget)
 {
     RsttoThumbnailBar *bar = RSTTO_THUMBNAIL_BAR(widget);
 
-    GtkAllocation view_allocation;
     GdkWindowAttr attributes;
     gint attributes_mask;
-    gint event_mask;
     gint border_width = 0;
 
     GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
@@ -493,7 +491,6 @@ rstto_thumbnail_bar_realize(GtkWidget *widget)
 static void
 rstto_thumbnail_bar_unrealize(GtkWidget *widget)
 {
-    RsttoThumbnailBar *bar = RSTTO_THUMBNAIL_BAR(widget);
 
     if (GTK_WIDGET_CLASS (parent_class)->unrealize)
         (* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
@@ -826,13 +823,16 @@ cb_rstto_thumbnail_bar_scroll_event (RsttoThumbnailBar *bar,
     {
         case GDK_SCROLL_UP:
         case GDK_SCROLL_LEFT:
-            rstto_navigator_jump_back(bar->priv->navigator);
+            bar->priv->auto_center = FALSE;
+            bar->priv->offset -= 30;
             break;
         case GDK_SCROLL_DOWN:
         case GDK_SCROLL_RIGHT:
-            rstto_navigator_jump_forward(bar->priv->navigator);
+            bar->priv->auto_center = FALSE;
+            bar->priv->offset += 30;
             break;
     }
+    gtk_widget_queue_resize(GTK_WIDGET(bar));
     return FALSE;
 
 }
